@@ -1,6 +1,5 @@
 import os
-from netlib import certutils, certffi
-import OpenSSL
+from netlib import certutils
 import tutils
 
 # class TestDNTree:
@@ -34,6 +33,7 @@ import tutils
 
 
 class TestCertStore:
+
     def test_create_explicit(self):
         with tutils.tmpdir() as d:
             ca = certutils.CertStore.from_store(d, "test")
@@ -42,7 +42,8 @@ class TestCertStore:
             ca2 = certutils.CertStore.from_store(d, "test")
             assert ca2.get_cert("foo", [])
 
-            assert ca.default_ca.get_serial_number() == ca2.default_ca.get_serial_number()
+            assert ca.default_ca.get_serial_number(
+            ) == ca2.default_ca.get_serial_number()
 
     def test_create_tmp(self):
         with tutils.tmpdir() as d:
@@ -56,13 +57,13 @@ class TestCertStore:
 
     def test_add_cert(self):
         with tutils.tmpdir() as d:
-            ca = certutils.CertStore.from_store(d, "test")
+            certutils.CertStore.from_store(d, "test")
 
     def test_sans(self):
         with tutils.tmpdir() as d:
             ca = certutils.CertStore.from_store(d, "test")
             c1 = ca.get_cert("foo.com", ["*.bar.com"])
-            c2 = ca.get_cert("foo.bar.com", [])
+            ca.get_cert("foo.bar.com", [])
             # assert c1 == c2
             c3 = ca.get_cert("bar.com", [])
             assert not c1 == c3
@@ -70,7 +71,7 @@ class TestCertStore:
     def test_sans_change(self):
         with tutils.tmpdir() as d:
             ca = certutils.CertStore.from_store(d, "test")
-            _ = ca.get_cert("foo.com", ["*.bar.com"])
+            ca.get_cert("foo.com", ["*.bar.com"])
             cert, key, chain_file = ca.get_cert("foo.bar.com", ["*.baz.com"])
             assert "*.baz.com" in cert.altnames
 
@@ -78,9 +79,10 @@ class TestCertStore:
         with tutils.tmpdir() as d:
             ca1 = certutils.CertStore.from_store(os.path.join(d, "ca1"), "test")
             ca2 = certutils.CertStore.from_store(os.path.join(d, "ca2"), "test")
-            assert not ca1.default_ca.get_serial_number() == ca2.default_ca.get_serial_number()
+            assert not ca1.default_ca.get_serial_number(
+            ) == ca2.default_ca.get_serial_number()
 
-            dc = ca2.get_cert("foo.com", [])
+            dc = ca2.get_cert("foo.com", ["sans.example.com"])
             dcp = os.path.join(d, "dc")
             f = open(dcp, "wb")
             f.write(dc[0].to_pem())
@@ -90,18 +92,9 @@ class TestCertStore:
             ret = ca1.get_cert("foo.com", [])
             assert ret[0].serial == dc[0].serial
 
-    def test_gen_pkey(self):
-        try:
-            with tutils.tmpdir() as d:
-                ca1 = certutils.CertStore.from_store(os.path.join(d, "ca1"), "test")
-                ca2 = certutils.CertStore.from_store(os.path.join(d, "ca2"), "test")
-                cert = ca1.get_cert("foo.com", [])
-                assert certffi.get_flags(ca2.gen_pkey(cert[0])) == 1
-        finally:
-            certffi.set_flags(ca2.default_privatekey, 0)
-
 
 class TestDummyCert:
+
     def test_with_ca(self):
         with tutils.tmpdir() as d:
             ca = certutils.CertStore.from_store(d, "test")
@@ -115,39 +108,41 @@ class TestDummyCert:
 
 
 class TestSSLCert:
+
     def test_simple(self):
         with open(tutils.test_data.path("data/text_cert"), "rb") as f:
             d = f.read()
-        c = certutils.SSLCert.from_pem(d)
-        assert c.cn == "google.com"
-        assert len(c.altnames) == 436
+        c1 = certutils.SSLCert.from_pem(d)
+        assert c1.cn == "google.com"
+        assert len(c1.altnames) == 436
 
         with open(tutils.test_data.path("data/text_cert_2"), "rb") as f:
             d = f.read()
-        c = certutils.SSLCert.from_pem(d)
-        assert c.cn == "www.inode.co.nz"
-        assert len(c.altnames) == 2
-        assert c.digest("sha1")
-        assert c.notbefore
-        assert c.notafter
-        assert c.subject
-        assert c.keyinfo == ("RSA", 2048)
-        assert c.serial
-        assert c.issuer
-        assert c.to_pem()
-        c.has_expired
+        c2 = certutils.SSLCert.from_pem(d)
+        assert c2.cn == "www.inode.co.nz"
+        assert len(c2.altnames) == 2
+        assert c2.digest("sha1")
+        assert c2.notbefore
+        assert c2.notafter
+        assert c2.subject
+        assert c2.keyinfo == ("RSA", 2048)
+        assert c2.serial
+        assert c2.issuer
+        assert c2.to_pem()
+        assert c2.has_expired is not None
+
+        assert not c1 == c2
+        assert c1 != c2
 
     def test_err_broken_sans(self):
         with open(tutils.test_data.path("data/text_cert_weird1"), "rb") as f:
             d = f.read()
         c = certutils.SSLCert.from_pem(d)
         # This breaks unless we ignore a decoding error.
-        c.altnames
+        assert c.altnames is not None
 
     def test_der(self):
         with open(tutils.test_data.path("data/dercert"), "rb") as f:
             d = f.read()
         s = certutils.SSLCert.from_der(d)
         assert s.cn
-
-
