@@ -1,6 +1,8 @@
-import cStringIO, textwrap, binascii
-from netlib import http, odict, tcp, test
-import tutils
+import cStringIO
+import textwrap
+import binascii
+from netlib import http, odict, tcp
+from . import tutils, tservers
 
 
 def test_httperror():
@@ -21,7 +23,11 @@ def test_read_chunked():
     h["transfer-encoding"] = ["chunked"]
     s = cStringIO.StringIO("1\r\na\r\n0\r\n")
 
-    tutils.raises("malformed chunked body", http.read_http_body, s, h, None, "GET", None, True)
+    tutils.raises(
+        "malformed chunked body",
+        http.read_http_body,
+        s, h, None, "GET", None, True
+    )
 
     s = cStringIO.StringIO("1\r\na\r\n0\r\n\r\n")
     assert http.read_http_body(s, h, None, "GET", None, True) == "a"
@@ -30,13 +36,25 @@ def test_read_chunked():
     assert http.read_http_body(s, h, None, "GET", None, True) == "a"
 
     s = cStringIO.StringIO("\r\n")
-    tutils.raises("closed prematurely", http.read_http_body, s, h, None, "GET", None, True)
+    tutils.raises(
+        "closed prematurely",
+        http.read_http_body,
+        s, h, None, "GET", None, True
+    )
 
     s = cStringIO.StringIO("1\r\nfoo")
-    tutils.raises("malformed chunked body", http.read_http_body, s, h, None, "GET", None, True)
+    tutils.raises(
+        "malformed chunked body",
+        http.read_http_body,
+        s, h, None, "GET", None, True
+    )
 
     s = cStringIO.StringIO("foo\r\nfoo")
-    tutils.raises(http.HttpError, http.read_http_body, s, h, None, "GET", None, True)
+    tutils.raises(
+        http.HttpError,
+        http.read_http_body,
+        s, h, None, "GET", None, True
+    )
 
     s = cStringIO.StringIO("5\r\naaaaa\r\n0\r\n\r\n")
     tutils.raises("too large", http.read_http_body, s, h, 2, "GET", None, True)
@@ -53,6 +71,7 @@ def test_connection_close():
     h["connection"] = ["close"]
     assert http.connection_close((1, 1), h)
 
+
 def test_get_header_tokens():
     h = odict.ODictCaseless()
     assert http.get_header_tokens(h, "foo") == []
@@ -69,11 +88,13 @@ def test_read_http_body_request():
     r = cStringIO.StringIO("testing")
     assert http.read_http_body(r, h, None, "GET", None, True) == ""
 
+
 def test_read_http_body_response():
     h = odict.ODictCaseless()
-    s = cStringIO.StringIO("testing")
+    s = tcp.Reader(cStringIO.StringIO("testing"))
     assert http.read_http_body(s, h, None, "GET", 200, False) == "testing"
 
+
 def test_read_http_body():
     # test default case
     h = odict.ODictCaseless()
@@ -84,17 +105,29 @@ def test_read_http_body():
     # test content length: invalid header
     h["content-length"] = ["foo"]
     s = cStringIO.StringIO("testing")
-    tutils.raises(http.HttpError, http.read_http_body, s, h, None, "GET", 200, False)
+    tutils.raises(
+        http.HttpError,
+        http.read_http_body,
+        s, h, None, "GET", 200, False
+    )
 
     # test content length: invalid header #2
     h["content-length"] = [-1]
     s = cStringIO.StringIO("testing")
-    tutils.raises(http.HttpError, http.read_http_body, s, h, None, "GET", 200, False)
+    tutils.raises(
+        http.HttpError,
+        http.read_http_body,
+        s, h, None, "GET", 200, False
+    )
 
     # test content length: content length > actual content
     h["content-length"] = [5]
     s = cStringIO.StringIO("testing")
-    tutils.raises(http.HttpError, http.read_http_body, s, h, 4, "GET", 200, False)
+    tutils.raises(
+        http.HttpError,
+        http.read_http_body,
+        s, h, 4, "GET", 200, False
+    )
 
     # test content length: content length < actual content
     s = cStringIO.StringIO("testing")
@@ -102,19 +135,24 @@ def test_read_http_body():
 
     # test no content length: limit > actual content
     h = odict.ODictCaseless()
-    s = cStringIO.StringIO("testing")
+    s = tcp.Reader(cStringIO.StringIO("testing"))
     assert len(http.read_http_body(s, h, 100, "GET", 200, False)) == 7
 
     # test no content length: limit < actual content
-    s = cStringIO.StringIO("testing")
-    tutils.raises(http.HttpError, http.read_http_body, s, h, 4, "GET", 200, False)
+    s = tcp.Reader(cStringIO.StringIO("testing"))
+    tutils.raises(
+        http.HttpError,
+        http.read_http_body,
+        s, h, 4, "GET", 200, False
+    )
 
     # test chunked
     h = odict.ODictCaseless()
     h["transfer-encoding"] = ["chunked"]
-    s = cStringIO.StringIO("5\r\naaaaa\r\n0\r\n\r\n")
+    s = tcp.Reader(cStringIO.StringIO("5\r\naaaaa\r\n0\r\n\r\n"))
     assert http.read_http_body(s, h, 100, "GET", 200, False) == "aaaaa"
 
+
 def test_expected_http_body_size():
     # gibber in the content-length field
     h = odict.ODictCaseless()
@@ -135,6 +173,7 @@ def test_expected_http_body_size():
     h = odict.ODictCaseless()
     assert http.expected_http_body_size(h, True, "GET", None) == 0
 
+
 def test_parse_http_protocol():
     assert http.parse_http_protocol("HTTP/1.1") == (1, 1)
     assert http.parse_http_protocol("HTTP/0.0") == (0, 0)
@@ -189,7 +228,9 @@ def test_parse_init_http():
     assert not http.parse_init_http("GET /test foo/1.1")
     assert not http.parse_init_http("GET /test\xc0 HTTP/1.1")
 
+
 class TestReadHeaders:
+
     def _read(self, data, verbatim=False):
         if not verbatim:
             data = textwrap.dedent(data)
@@ -237,36 +278,44 @@ class TestReadHeaders:
 
 
 class NoContentLengthHTTPHandler(tcp.BaseHandler):
+
     def handle(self):
         self.wfile.write("HTTP/1.1 200 OK\r\n\r\nbar\r\n\r\n")
         self.wfile.flush()
 
 
-class TestReadResponseNoContentLength(test.ServerTestBase):
+class TestReadResponseNoContentLength(tservers.ServerTestBase):
     handler = NoContentLengthHTTPHandler
 
     def test_no_content_length(self):
         c = tcp.TCPClient(("127.0.0.1", self.port))
         c.connect()
-        httpversion, code, msg, headers, content = http.read_response(c.rfile, "GET", None)
-        assert content == "bar\r\n\r\n"
+        resp = http.read_response(c.rfile, "GET", None)
+        assert resp.content == "bar\r\n\r\n"
+
 
 def test_read_response():
     def tst(data, method, limit, include_body=True):
         data = textwrap.dedent(data)
         r = cStringIO.StringIO(data)
-        return  http.read_response(r, method, limit, include_body=include_body)
+        return http.read_response(
+            r, method, limit, include_body=include_body
+        )
 
     tutils.raises("server disconnect", tst, "", "GET", None)
     tutils.raises("invalid server response", tst, "foo", "GET", None)
     data = """
         HTTP/1.1 200 OK
     """
-    assert tst(data, "GET", None) == ((1, 1), 200, 'OK', odict.ODictCaseless(), '')
+    assert tst(data, "GET", None) == (
+        (1, 1), 200, 'OK', odict.ODictCaseless(), ''
+    )
     data = """
         HTTP/1.1 200
     """
-    assert tst(data, "GET", None) == ((1, 1), 200, '', odict.ODictCaseless(), '')
+    assert tst(data, "GET", None) == (
+        (1, 1), 200, '', odict.ODictCaseless(), ''
+    )
     data = """
         HTTP/x 200 OK
     """
@@ -281,7 +330,9 @@ def test_read_response():
 
         HTTP/1.1 200 OK
     """
-    assert tst(data, "GET", None) == ((1, 1), 100, 'CONTINUE', odict.ODictCaseless(), '')
+    assert tst(data, "GET", None) == (
+        (1, 1), 100, 'CONTINUE', odict.ODictCaseless(), ''
+    )
 
     data = """
         HTTP/1.1 200 OK
@@ -306,7 +357,7 @@ def test_read_response():
 
         foo
     """
-    assert tst(data, "GET", None, include_body=False)[4] == None
+    assert tst(data, "GET", None, include_body=False)[4] is None
 
 
 def test_parse_url():
@@ -325,6 +376,12 @@ def test_parse_url():
     assert po == 80
     assert pa == "/bar"
 
+    s, h, po, pa = http.parse_url("http://user:pass@foo/bar")
+    assert s == "http"
+    assert h == "foo"
+    assert po == 80
+    assert pa == "/bar"
+
     s, h, po, pa = http.parse_url("http://foo")
     assert pa == "/"
 
@@ -345,11 +402,90 @@ def test_parse_url():
     # Invalid IPv6 URL - see http://www.ietf.org/rfc/rfc2732.txt
     assert not http.parse_url('http://lo[calhost')
 
+
 def test_parse_http_basic_auth():
     vals = ("basic", "foo", "bar")
-    assert http.parse_http_basic_auth(http.assemble_http_basic_auth(*vals)) == vals
+    assert http.parse_http_basic_auth(
+        http.assemble_http_basic_auth(*vals)
+    ) == vals
     assert not http.parse_http_basic_auth("")
     assert not http.parse_http_basic_auth("foo bar")
     v = "basic " + binascii.b2a_base64("foo")
     assert not http.parse_http_basic_auth(v)
 
+
+def test_get_request_line():
+    r = cStringIO.StringIO("\nfoo")
+    assert http.get_request_line(r) == "foo"
+    assert not http.get_request_line(r)
+
+
+class TestReadRequest():
+
+    def tst(self, data, **kwargs):
+        r = cStringIO.StringIO(data)
+        return http.read_request(r, **kwargs)
+
+    def test_invalid(self):
+        tutils.raises(
+            "bad http request",
+            self.tst,
+            "xxx"
+        )
+        tutils.raises(
+            "bad http request line",
+            self.tst,
+            "get /\xff HTTP/1.1"
+        )
+        tutils.raises(
+            "invalid headers",
+            self.tst,
+            "get / HTTP/1.1\r\nfoo"
+        )
+        tutils.raises(
+            tcp.NetLibDisconnect,
+            self.tst,
+            "\r\n"
+        )
+
+    def test_asterisk_form_in(self):
+        v = self.tst("OPTIONS * HTTP/1.1")
+        assert v.form_in == "relative"
+        assert v.method == "OPTIONS"
+
+    def test_absolute_form_in(self):
+        tutils.raises(
+            "Bad HTTP request line",
+            self.tst,
+            "GET oops-no-protocol.com HTTP/1.1"
+        )
+        v = self.tst("GET http://address:22/ HTTP/1.1")
+        assert v.form_in == "absolute"
+        assert v.port == 22
+        assert v.host == "address"
+        assert v.scheme == "http"
+
+    def test_connect(self):
+        tutils.raises(
+            "Bad HTTP request line",
+            self.tst,
+            "CONNECT oops-no-port.com HTTP/1.1"
+        )
+        v = self.tst("CONNECT foo.com:443 HTTP/1.1")
+        assert v.form_in == "authority"
+        assert v.method == "CONNECT"
+        assert v.port == 443
+        assert v.host == "foo.com"
+
+    def test_expect(self):
+        w = cStringIO.StringIO()
+        r = cStringIO.StringIO(
+            "GET / HTTP/1.1\r\n"
+            "Content-Length: 3\r\n"
+            "Expect: 100-continue\r\n\r\n"
+            "foobar",
+        )
+        v = http.read_request(r, wfile=w)
+        assert w.getvalue() == "HTTP/1.1 100 Continue\r\n\r\n"
+        assert v.content == "foo"
+        assert r.read(3) == "bar"
